@@ -1,13 +1,13 @@
-############################################################################################
+###############################################################################
 ## package 'secr'
 ## D.design.MS.R
 ## Prepare density design matrix
 ##
 ## force levels=sessionlevels for dframe$session 2010 02 25
 ## temporary raise error if mask covariates and multiple sessions
-############################################################################################
+###############################################################################
 ## NOTE does not standardize sessioncov, maskcov
-############################################################################################
+###############################################################################
 
 D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL) {
 
@@ -39,10 +39,10 @@ D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL)
     }
     #######################################
 
-    MS <- length(sessionlevels) > 1  ## !inherits(mask, 'mask')
-    if (MS) nmask <- max(sapply(mask, nrow))
+    ## 2011-05-04
+    # MS <- length(sessionlevels) > 1  ## !inherits(mask, 'mask')
+    if (ms(mask)) nmask <- max(sapply(mask, nrow))
     else nmask <- nrow(mask)
-
     vars  <- all.vars(Dmodel)
 
     ##  Always conform to the global number of groups 2009 06 23
@@ -57,7 +57,7 @@ D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL)
         pad1 (stdfn(msk[,colnum]), nmask)
     }
 
-    if (MS) {
+    if (ms(mask)) {
         x <- lapply(mask, getcol, 1)
         y <- lapply(mask, getcol, 2)   ## bug 2009 09 04: changed 1 -> 2
     }
@@ -66,14 +66,20 @@ D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL)
         y <- getcol(mask,2)
     }
 
+    ## 2011-05-04
+#    dframe <- as.data.frame( list (
+#      x = rep(as.vector(unlist(x)), ngrp),
+#      y = rep(as.vector(unlist(y)), ngrp)
+#    ))
+
     dframe <- as.data.frame( list (
-      x = rep(as.vector(unlist(x)), ngrp),
-      y = rep(as.vector(unlist(y)), ngrp)
+      x = rep(as.vector(unlist(x)), ngrp * R),
+      y = rep(as.vector(unlist(y)), ngrp * R)
     ))
 
     if ('g' %in% vars) {
         if (length(grps)<1)
-            stop("no groups specified")
+            stop ("no groups specified")
         dframe$g <- insertdim(factor(grps), 2, dims)
     }
 
@@ -92,6 +98,7 @@ D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL)
     if ('session' %in% vars) {
        dframe$session <- insertdim(factor(sessionlevels, levels=sessionlevels), 3, dims)
     }
+
     if ('Session' %in% vars) {
        dframe$Session <- insertdim(0:(R-1), 3, dims)
     }
@@ -104,16 +111,17 @@ D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL)
     }
 
     if ((!is.null(covariates(mask))) & (length(vars>0))) {
-        if (MS)
-            stop ("D.designdata does not yet handle mask covariates for multiple sessions")
+        if (ms(mask))
+            stop ("D.designdata does not yet handle mask covariates ",
+                  "for multiple sessions")
         findvars (covariates(mask), vars, 1, std = FALSE)
     }
 
     if (length(vars)>0)
-        stop (paste(paste(vars,collapse=','),"not found"))
+        stop (paste(vars,collapse=','), "not found")
 
     attr(dframe, 'dimD') <- c(nmask, ngrp, R)
     dframe
 }
-############################################################################################
+###############################################################################
 
